@@ -34,6 +34,17 @@ class _KeywordFoundFromAst(object):
         self.completion_context = completion_context
         self.completion_item_kind = completion_item_kind
 
+    @property
+    def library_name(self):
+        return None
+
+    @property
+    def resource_name(self):
+        from robotframework_ls import uris
+
+        uri = self.completion_context.doc.uri
+        return os.path.splitext(os.path.basename(uris.to_fs_path(uri)))[0]
+
     @property
     def docs_format(self):
         return "markdown"
@@ -100,6 +111,14 @@ class _KeywordFoundFromLibrary(object):
         self.completion_context = completion_context
         self.completion_item_kind = completion_item_kind
 
+    @property
+    def library_name(self):
+        return self._library_doc.name
+
+    @property
+    def resource_name(self):
+        return None
+
     @property
     @cache.instance_cache
     def source(self):
@@ -238,48 +257,9 @@ def _collect_resource_imports_keywords(completion_context, collector):
     """
     :param CompletionContext completion_context:
     """
-    from robotframework_ls import uris
-
-    # Get keywords from resources
-    resource_imports = completion_context.get_resource_imports()
-    for resource_import in resource_imports:
-        for token in resource_import.tokens:
-            if token.type == token.NAME:
-                parts = []
-                for v in token.tokenize_variables():
-                    if v.type == v.NAME:
-                        parts.append(str(v))
-
-                    elif v.type == v.VARIABLE:
-                        # Resolve variable from config
-                        v = str(v)
-                        if v.startswith("${") and v.endswith("}"):
-                            v = v[2:-1]
-                            parts.append(completion_context.convert_robot_variable(v))
-                        else:
-                            log.info("Cannot resolve variable: %s", v)
-
-                resource_path = "".join(parts)
-                if not os.path.isabs(resource_path):
-                    # It's a relative resource, resolve its location based on the
-                    # current file.
-                    resource_path = os.path.join(
-                        os.path.dirname(completion_context.doc.path), resource_path
-                    )
-
-                ws = completion_context.workspace
-                if not os.path.exists(resource_path):
-                    log.info("Resource not found: %s", resource_path)
-                    continue
-
-                doc_uri = uris.from_fs_path(resource_path)
-
-                resource_doc = ws.get_document(doc_uri, create=False)
-                if resource_doc is None:
-                    resource_doc = ws.create_untracked_document(doc_uri)
-
-                new_ctx = completion_context.create_copy(resource_doc)
-                _collect_following_imports(new_ctx, collector)
+    for resource_doc in completion_context.iter_imports_docs():
+        new_ctx = completion_context.create_copy(resource_doc)
+        _collect_following_imports(new_ctx, collector)
 
 
 def _collect_following_imports(completion_context, collector):
@@ -322,6 +302,12 @@ class IKeywordFound(object):
     col_offset = -1
     end_col_offset = -1
 
+    # If it's a library, this is the name of the library.
+    library_name = None
+
+    # If it's a resource, this is the basename of the resource without the extension.
+    resource_name = None
+
 
 class ICollector(object):
     def accepts(self, keyword_name):
