@@ -4,16 +4,32 @@ log = get_logger(__name__)
 
 
 class _Collector(object):
-    def __init__(self, selection, token, matcher):
-        self.matcher = matcher
+    def __init__(self, selection, token):
+        from robotframework_ls.impl.string_matcher import RobotStringMatcher
+        from robotframework_ls.impl.string_matcher import (
+            build_matchers_with_resource_or_library_scope,
+        )
+
+        token_str = token.value
+
         self.completion_items = []
         self.selection = selection
         self.token = token
 
-    def accepts(self, keyword_name):
-        return self.matcher.accepts(keyword_name)
+        self._matcher = RobotStringMatcher(token_str)
+        self._scope_matchers = build_matchers_with_resource_or_library_scope(token_str)
 
-    def _create_completion_item_from_keyword(self, keyword_found, selection, token):
+    def accepts(self, keyword_name):
+        if self._matcher.accepts_keyword_name(keyword_name):
+            return True
+        for matcher in self._scope_matchers:
+            if matcher.accepts_keyword_name(keyword_name):
+                return True
+        return False
+
+    def _create_completion_item_from_keyword(
+        self, keyword_found, selection, token, col_delta=0
+    ):
         """
         :param IKeywordFound keyword_found:
         :param selection:
@@ -32,11 +48,18 @@ class _Collector(object):
         text = label
 
         for i, arg in enumerate(keyword_found.keyword_args):
+            arg = arg.replace("$", "\\$").replace("{", "").replace("}", "")
+            if arg.startswith("**"):
+                arg = "&" + arg[2:]
+
+            elif arg.startswith("*"):
+                arg = "@" + arg[1:]
+
             text += "    ${%s:%s}" % (i + 1, arg)
 
         text_edit = TextEdit(
             Range(
-                start=Position(selection.line, token.col_offset),
+                start=Position(selection.line, token.col_offset + col_delta),
                 end=Position(selection.line, token.end_col_offset),
             ),
             text,
@@ -57,8 +80,18 @@ class _Collector(object):
         ).to_dict()
 
     def on_keyword(self, keyword_found):
+        col_delta = 0
+        if not self._matcher.accepts_keyword_name(keyword_found.keyword_name):
+            for matcher in self._scope_matchers:
+                if matcher.accepts_keyword(keyword_found):
+                    # +1 for the dot
+                    col_delta = len(matcher.resource_or_library_name) + 1
+                    break
+            else:
+                return  # i.e.: don't add completion
+
         item = self._create_completion_item_from_keyword(
-            keyword_found, self.selection, self.token
+            keyword_found, self.selection, self.token, col_delta=col_delta
         )
 
         self.completion_items.append(item)
@@ -68,20 +101,16 @@ def complete(completion_context):
     """
     :param CompletionContext completion_context:
     """
-    from robotframework_ls.impl.string_matcher import RobotStringMatcher
     from robotframework_ls.impl.collect_keywords import collect_keywords
     from robotframework_ls.impl import ast_utils
 
     token_info = completion_context.get_current_token()
-    if token_info is not None and ast_utils.is_keyword_name_location(
-        token_info.node, token_info.token
-    ):
-        token = token_info.token
-        collector = _Collector(
-            completion_context.sel, token, RobotStringMatcher(token.value)
-        )
-        collect_keywords(completion_context, collector)
+    if token_info is not None:
+        token = ast_utils.get_keyword_name_token(token_info.node, token_info.token)
+        if token is not None:
+            collector = _Collector(completion_context.sel, token)
+            collect_keywords(completion_context, collector)
 
-        return collector.completion_items
+            return collector.completion_items
 
     return []
